"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var q = require("q");
var logger_1 = require("./logger");
var pact_util_1 = require("./pact-util");
var pact_standalone_1 = require("./pact-standalone");
var _ = require("underscore");
var checkTypes = require("check-types");
var CanDeploy = (function () {
    function CanDeploy(options) {
        this.__argMapping = {
            "pacticipant": "--pacticipant",
            "pacticipantVersion": "--version",
            "latest": "--latest",
            "to": "--to",
            "pactBroker": "--broker-base-url",
            "pactBrokerUsername": "--broker-username",
            "pactBrokerPassword": "--broker-password",
            "output": "--output",
            "verbose": "--verbose",
            "retryWhileUnknown": "--retry-while-unknown",
            "retryInterval": "--retry-interval",
        };
        options = options || {};
        options.timeout = options.timeout || 60000;
        options.pacticipant = options.pacticipant || [];
        options.pacticipantVersion = options.pacticipantVersion || [];
        checkTypes.assert.nonEmptyString(options.pactBroker, "Must provide the pactBroker argument");
        checkTypes.assert.arrayLike(options.pacticipant, "Must provide the pacticipant argument");
        checkTypes.assert.arrayLike(options.pacticipantVersion, "Must provide the version argument");
        checkTypes.assert.equal(options.pacticipant.length, options.pacticipantVersion.length, "Must provide the same number of pacticipant and version");
        checkTypes.assert.nonEmptyArray(options.pacticipant, "Pacticipant array must not be empty");
        checkTypes.assert.nonEmptyArray(options.pacticipantVersion, "Version array must not be empty");
        if (options.pactBroker) {
            checkTypes.assert.string(options.pactBroker);
        }
        if (options.pactBrokerUsername) {
            checkTypes.assert.string(options.pactBrokerUsername);
        }
        if (options.pactBrokerPassword) {
            checkTypes.assert.string(options.pactBrokerPassword);
        }
        if ((options.pactBrokerUsername && !options.pactBrokerPassword) || (options.pactBrokerPassword && !options.pactBrokerUsername)) {
            throw new Error("Must provide both Pact Broker username and password. None needed if authentication on Broker is disabled.");
        }
        this.options = options;
    }
    CanDeploy.convertForSpawnBinary = function (options) {
        return _.flatten(_.zip(_.map(options.pacticipant, function (x) { return ({ pacticipant: x }); }), _.map(options.pacticipantVersion, function (x) { return ({ version: x }); })))
            .concat([_.omit(options, "pacticipant", "pacticipantVersion")]);
    };
    CanDeploy.prototype.canDeploy = function () {
        logger_1.default.info("Asking broker at " + this.options.pactBroker + " if it is possible to deploy");
        var deferred = q.defer();
        var instance = pact_util_1.default.spawnBinary(pact_standalone_1.default.brokerPath + " can-i-deploy", CanDeploy.convertForSpawnBinary(this.options), this.__argMapping);
        var output = [];
        instance.stdout.on("data", function (l) { return output.push(l); });
        instance.stderr.on("data", function (l) { return output.push(l); });
        instance.once("close", function (code) {
            var o = output.join("\n");
            var success = /All verification results are published and successful/igm.exec(o);
            if (code !== 0 || !success) {
                logger_1.default.error("can-i-deploy did not return success message:\n" + o);
                return deferred.reject(new Error(o));
            }
            logger_1.default.info(o);
            return deferred.resolve();
        });
        return deferred.promise
            .timeout(this.options.timeout, "Timeout waiting for verification process to complete (PID: " + instance.pid + ")");
    };
    return CanDeploy;
}());
exports.CanDeploy = CanDeploy;
exports.default = (function (options) { return new CanDeploy(options); });
//# sourceMappingURL=can-deploy.js.map